from flask import Flask, request, jsonify
import traceback
import json
from app import app
from app.agoda import agoda_reviews_scrapper
from app.booking import booking_review_scrapper
from app.hotels import hotels_review_scrapper
from app.trip import trip_review_scrapper
import concurrent.futures
from concurrent.futures import ThreadPoolExecutor, as_completed

@app.route("/")
def root():
    message = {"message":"Welcome to Global Repute Score API !!"}
    return jsonify(message)

@app.route("/get_grs",methods=["POST"])
def get_grs():
    request_data = request.get_json()
    hotel_id = request_data['hotel_id']
    hotel_name = request_data['hotel_name']
    hotel_lat = request_data['lat']
    hotel_long = request_data['long']
    city = request_data['city']

    grs_score=0
    grs_total_reviews=0
    grs_count = 0
    booking_review_score=0
    booking_review_count =0
    hotels_review_score = 0
    hotels_review_count = 0
    agoda_review_score=0
    agoda_review_count=0
    trip_review_score=0
    trip_review_count=0

    with concurrent.futures.ThreadPoolExecutor(max_workers=4) as executor:
        futures = []
        futures.append(executor.submit(agoda_reviews_scrapper.getReviewsFromAgoda,hotel_id,hotel_name,city,hotel_lat,hotel_long))
        futures.append(executor.submit(booking_review_scrapper.getReviewFromBooking,hotel_id,hotel_name,city,hotel_lat,hotel_long))
        futures.append(executor.submit(hotels_review_scrapper.getReviewsFromHotels,hotel_id, hotel_name, city, hotel_lat, hotel_long))
        futures.append(executor.submit(trip_review_scrapper.getReviewsFromTrip,hotel_id,hotel_name,city,hotel_lat,hotel_long))

        for future in concurrent.futures.as_completed(futures):
            result = future.result(future)
            print("Result : "+str(result))
            if result:
                if 'agoda_review_score' in result:
                    agoda_reviews = result
                    if agoda_reviews:
                        if agoda_reviews['agoda_review_score'] is not None and agoda_reviews['agoda_review_score'] > 0:
                            agoda_review_score = agoda_reviews['agoda_review_score']
                            grs_score = grs_score + agoda_reviews['agoda_review_score']
                            grs_count = grs_count + 1
                        if agoda_reviews['agoda_total_reviews'] is not None and agoda_reviews[
                            'agoda_total_reviews'] > 0:
                            agoda_review_count = agoda_reviews['agoda_total_reviews']
                            grs_total_reviews = grs_total_reviews + agoda_reviews['agoda_total_reviews']

                elif 'booking_review_score' in result:
                    booking_reviews = result
                    if booking_reviews:
                        if booking_reviews['booking_review_score'] is not None and booking_reviews[
                            'booking_review_score'] > 0:
                            booking_review_score = booking_reviews['booking_review_score']
                            grs_score = grs_score + booking_reviews['booking_review_score']
                            grs_count = grs_count + 1
                        if booking_reviews['booking_total_reviews'] is not None and booking_reviews[
                            'booking_total_reviews'] > 0:
                            booking_review_count = booking_reviews['booking_total_reviews']
                            grs_total_reviews = grs_total_reviews + booking_reviews['booking_total_reviews']

                elif 'hotels_review_score' in result:
                    hotels_reviews = result
                    if hotels_reviews:
                        if hotels_reviews['hotels_review_score'] is not None and hotels_reviews[
                            'hotels_review_score'] > 0:
                            hotels_review_score = hotels_reviews['hotels_review_score']
                            grs_score = grs_score + hotels_reviews['hotels_review_score']
                            grs_count = grs_count + 1
                        if hotels_reviews['hotels_total_reviews'] is not None and hotels_reviews[
                            'hotels_total_reviews'] > 0:
                            hotels_review_count = hotels_reviews['hotels_total_reviews']
                            grs_total_reviews = grs_total_reviews + hotels_reviews['hotels_total_reviews']

                elif 'trip_review_score' in result:
                    trip_reviews = result
                    if trip_reviews:
                        if trip_reviews['trip_review_score'] is not None and trip_reviews['trip_review_score'] > 0:
                            trip_review_score = trip_reviews['trip_review_score']
                            grs_score = grs_score + trip_reviews['trip_review_score']
                            grs_count = grs_count + 1
                        if trip_reviews['trip_total_reviews'] is not None and trip_reviews['trip_total_reviews'] > 0:
                            trip_review_count = trip_reviews['trip_total_reviews']
                            grs_total_reviews = grs_total_reviews + trip_reviews['trip_total_reviews']


    avg_grs_score = 0
    adjective=''
    if grs_score > 0:
        avg_grs_score = round(float(grs_score / grs_count),1)
        adjective = getReviewAdjectives(avg_grs_score)
    data= {
        'grs':avg_grs_score,
        'adjective':adjective,
        'total_reviews':grs_total_reviews,
        # 'booking_review_score':booking_review_score,
        # 'booking_review_count':booking_review_count,
        # 'hotels_review_score':hotels_review_score,
        # 'hotels_review_count':hotels_review_count,
        # 'agoda_review_score':agoda_review_score,
        # 'agoda_review_count':agoda_review_count,
        # 'trip_review_score':trip_review_score,
        # 'trip_review_count':trip_review_count,
    }
    return jsonify({"Message":"GRS calculated","data":data,"status_code":200})

def getReviewAdjectives(avg_grs_score):
    adjective=None

    if avg_grs_score >=3 and avg_grs_score<3.5:
        adjective='OK'

    if avg_grs_score >=3.5 and avg_grs_score<4:
        adjective='Good'

    if avg_grs_score >=4 and avg_grs_score<4.5:
        adjective='Very Good'

    if avg_grs_score >=4.5 and avg_grs_score<5:
        adjective='Wonderful'

    if avg_grs_score ==5:
        adjective='Exceptional'

    return adjective
